@@ -21,7 +21,7 @@ Example:
 """
 
 import re, pexpect, sys, time
-from byexample.common import log, build_exception_msg
+from byexample.common import log, build_exception_msg, constant
 from byexample.parser import ExampleParser
 from byexample.finder import ExampleFinder
 from byexample.runner import ExampleRunner, PexepctMixin
@@ -29,6 +29,7 @@ from byexample.runner import ExampleRunner, PexepctMixin
 class PythonPromptFinder(ExampleFinder):
     target = 'python-prompt'
 
+    @constant
     def example_regex(self):
         return re.compile(r'''
             # Snippet consists of a PS1 line >>>
@@ -128,21 +129,30 @@ _EXCEPTION_RE = re.compile(r"""
 # : : : : : : : : : : : : : : : : : : : : : : : : : : : : : : : : : : : : : : #
 ###############################################################################
 
+def _example_options_string_regex_for(compatibility_mode):
+    # anything of the form:
+    if compatibility_mode:
+        #   #  doctest:  +FOO -BAR +ZAZ=42
+        # or
+        #   #  byexample:  +FOO -BAR +ZAZ=42
+        keyword = r'(?:doctest|byexample)'
+    else:
+        #   #  byexample:  +FOO -BAR +ZAZ=42
+        keyword = r'byexample'
+
+    return re.compile(r'#\s*%s:\s*([^\n\'"]*)$' % keyword,
+                                                re.MULTILINE)
 class PythonParser(ExampleParser):
     language = 'python'
 
+    # make this to cache the regexs
     _blankline_tag_re = re.compile(r'^<BLANKLINE>$', re.MULTILINE|re.DOTALL)
+    _opts_re_for_noncomp = _example_options_string_regex_for(False)
+    _opts_re_for_comp = _example_options_string_regex_for(True)
 
     def example_options_string_regex(self):
-        # anything of the form:
-        #   #  byexample:  +FOO -BAR +ZAZ=42
-        if self.compatibility_mode:
-            keyword = r'(?:doctest|byexample)'
-        else:
-            keyword = r'byexample'
-
-        return re.compile(r'#\s*%s:\s*([^\n\'"]*)$' % keyword,
-                                                    re.MULTILINE)
+        return self._opts_re_for_comp if self.compatibility_mode \
+                else self._opts_re_for_noncomp
 
     def extend_option_parser(self, parser):
         '''
@@ -452,7 +462,7 @@ del _byexample_pprint
         self.cmd = self._get_cmd(pretty_print)
 
         # run!
-        self._spawn_interpreter()
+        self._spawn_interpreter(delaybeforesend=options['delaybeforesend'])
 
     def shutdown(self):
         self._shutdown_interpreter()
