@@ -1,5 +1,5 @@
 import collections, re, shlex, argparse
-from .common import log, build_exception_msg, tohuman
+from .common import log, build_exception_msg, tohuman, constant
 from .options import OptionParser, UnrecognizedOption
 
 Example = collections.namedtuple('Example', ['runner',
@@ -63,6 +63,7 @@ class ExampleParser(object):
         '''
         return parser
 
+    @constant
     def capture_tag_regex(self):
         '''
         Return a regular expression to match a 'capture tag'.
@@ -71,12 +72,51 @@ class ExampleParser(object):
         '''
         return re.compile(r"<(?P<name>(?:\w|-|\.)+)>")
 
-    def whitespace_non_compiled_regex(self):
-        return r'\s+'
+    @constant
+    def leading_optional_whitespace_regex(self):
+        return re.compile(r'\A\s*', re.MULTILINE | re.DOTALL)
+
+    @constant
+    def leading_single_whitespace_regex(self):
+        return re.compile(r'\A\s', re.MULTILINE | re.DOTALL)
+
+    @constant
+    def one_or_more_whitespace_regex(self):
+        return re.compile(r'\s+', re.MULTILINE | re.DOTALL)
+
+    @constant
+    def trailing_whitespace_regex(self):
+        return re.compile(r'\s+\Z', re.MULTILINE | re.DOTALL)
+
+    @constant
+    def trailing_single_whitespace_regex(self):
+        return re.compile(r'\s\Z', re.MULTILINE | re.DOTALL)
+
+    @constant
+    def trailing_newlines_regex(self):
+        return re.compile(r'\n+\Z', re.MULTILINE | re.DOTALL)
 
     def ellipsis_marker(self):
         return '...'
 
+    def do_not_begin_with_whitespace_regex_str(self):
+        return r'(?!\s)'
+
+    def do_not_end_with_whitespace_regex_str(self):
+        return r'(?<!\s)'
+
+    def do_not_end_with_newline_regex_str(self):
+        return r'(?<!\n)'
+
+    def trailing_optional_whitespace_regex_str(self):
+        return r"\s*\Z"
+
+    def trailing_optional_newline_regex_str(self):
+        return r'\n*\Z'
+
+    def non_capture_anything_regex_str(self):
+        return r"(?:.*?)"
+
     def process_snippet_and_expected(self, snippet, expected, where):
         r'''
         Process the snippet code and the expected output.
@@ -287,7 +327,7 @@ class ExampleParser(object):
             # By doing this, we need to re calculate their charnos too
             _es = [exprs[0]]
             _cs = [charnos[0]]
-            leading_ws_re = re.compile(r'\A\s*', re.MULTILINE | re.DOTALL)
+            leading_ws_re = self.leading_optional_whitespace_regex()
             for c, e in zip(charnos[1:], exprs[1:]):
                 olen = len(e)
                 lstripped = leading_ws_re.sub('', e)
@@ -311,7 +351,7 @@ class ExampleParser(object):
             # them by a \s+ regex
             # Because we will be mixing regexs with literals, it is time
             # to build safe literals and count the 'real counts'
-            any_ws_re = re.compile(r'\s+', re.MULTILINE | re.DOTALL)
+            any_ws_re = self.one_or_more_whitespace_regex()
 
             _rcs = []
             _es = []
@@ -579,8 +619,9 @@ class ExampleParser(object):
         # posible empty line at the end later
         # when normalize_whitespace == True, ignore remove all trailing
         # whitespaces
-        expected = re.sub(r'\s+\Z' if normalize_whitespace else r'\n+\Z',
-                          '', expected)
+        trailing_re = self.trailing_whitespace_regex() if normalize_whitespace \
+                      else self.trailing_newlines_regex()
+        expected = trailing_re.sub('', expected)
 
         charno = 0
         names_seen = []
@@ -619,7 +660,7 @@ class ExampleParser(object):
 
                 if name == self.ellipsis_marker():
                     # capture anything (non-greedy)
-                    regex = r"(?:.*?)"
+                    regex = self.non_capture_anything_regex_str()
 
                 else:
                     if name in names_seen:
@@ -637,17 +678,21 @@ class ExampleParser(object):
                 # space if the previous regex already matches that
                 # do the same for the trailing space and next regex
                 if normalize_whitespace:
-                    if re.search(r'\s\Z', pre_capture, re.MULTILINE | re.DOTALL):
-                        regex = r'(?!\s)' + regex  # do not begin with a whitespace
+                    if self.trailing_single_whitespace_regex().search(pre_capture):
+                        regex = self.do_not_begin_with_whitespace_regex_str() + \
+                                regex
 
-                    if re.search(r'\A\s', post_capture, re.MULTILINE | re.DOTALL):
-                        regex = regex + r'(?<!\s)' # do not end with a whitespace
+                    if self.leading_single_whitespace_regex().search(post_capture):
+                        regex = regex + \
+                                self.do_not_end_with_whitespace_regex_str()
 
                 if not post_capture:
                     if normalize_whitespace:
-                        regex = regex + r'(?<!\s)' # do not end with a whitespace
+                        regex = regex + \
+                                self.do_not_end_with_whitespace_regex_str()
                     else:
-                        regex = regex + r'(?<!\n)' # do not end with a newline
+                        regex = regex + \
+                                self.do_not_end_with_newline_regex_str()
 
                 regexs.append(regex)
                 charnos.append(charno)
@@ -666,7 +711,10 @@ class ExampleParser(object):
 
         # the end: ignore any trailing new line (trailing whitespace if
         # normalize_whitespace == True)
-        regexs.append(r"\s*\Z" if normalize_whitespace else r'\n*\Z')
+        trailing_re_str = self.trailing_optional_whitespace_regex_str() \
+                               if normalize_whitespace \
+                               else self.trailing_optional_newline_regex_str()
+        regexs.append(trailing_re_str)
         charnos.append(charno)
         rcounts.append(0)
 
