@@ -151,7 +151,7 @@ class PexepctMixin(object):
 
         # remove any other 'prompt' if any
         if self.any_PS_re:
-            out = re.sub(self.any_PS_re, '', out)
+            out = self.any_PS_re.sub('', out)
 
         # uniform the new line endings (aka universal new lines)
         out = self._universal_new_lines(out)
@@ -159,13 +159,7 @@ class PexepctMixin(object):
         return out
 
     def _universal_new_lines(self, out):
-        out = re.sub(r'\r\n', r'\n', out)
-
-        # TODO: is this ok?
-        if out and not out.endswith('\n'):
-            out += '\n'
-
-        return out
+        return out.replace('\r\n', '\n').replace('\r', '\n')
 
     def _set_cooked_mode(self, state): # pragma: no cover
         # code borrowed from ptyprocess/ptyprocess.py, _setecho, and
